/**************************************************************************
**       Title: histogram intersection kernel
**    $RCSfile: Kernel_HISTINTERSECT.hh,v $
**   $Revision: 1.2 $$Name:  $
**       $Date: 2005/10/26 07:03:27 $
**   Copyright: LGPL $Author: ronneber $
** Description:
**
**    
**
**-------------------------------------------------------------------------
**
** 
**  $Log:  
**
**
**
**************************************************************************/

#ifndef KERNEL_HISTINTERSECT_HH
#define KERNEL_HISTINTERSECT_HH

// std includes
#include <map>
#include <algorithm>
#include <set>
#include <string>

// libsvmtl includes
#include "ProgressReporter.hh"

// requirements of template parameters
#include "svt_check/RequireStData.hh"

namespace svt
{
  /*======================================================================*/
  /*!
   *  \class Kernel_HISTINTERSECT
   *  \ingroup kernel
   *  \brief The Kernel_HISTINTERSECT class specifies a HISTINTERSECT kernel function: min(u'(i),v(i))
   */
  /*======================================================================*/
  class Kernel_HISTINTERSECT
  {
  public:
    Kernel_HISTINTERSECT()
          {};
    
    ~Kernel_HISTINTERSECT()
          {};
        
    template< typename ForwardIter, typename Accessor>
    void updateCache( const ForwardIter& fvBegin,  
                      const ForwardIter& fvEnd,
                      Accessor accessor,
                      ProgressReporter* pr = 0) const
          {
            //  nothing to do here
          }
    
    // new updateCache-Syntax:    
    template< typename ForwardIter1, typename Accessor1,
              typename ForwardIter2, typename Accessor2 >
    void updateCache( const ForwardIter1& fvBegin1,  
                      const ForwardIter1& fvEnd1,
                      Accessor1 accessor1,
                      const ForwardIter2& fvBegin2,  
                      const ForwardIter2& fvEnd2,
                      Accessor2 accessor2,
                      ProgressReporter* pr = 0) const
          {
            // nothing to do here
          }
    
    void clearCache() const
          {
            //  nothing to do here
          }
            
    template< typename FV>
    double k_function( const FV& x, const FV& y) const
          {
	      double temp = 0.0;
	      for (size_t i=0;i<x.size();i++)
	      {
		  temp += std::min (x[i],y[i]);
	      }
            return temp;
          };    

    template<typename STDATA>
    void loadParameters( STDATA& stData)
          {};
    
    template<typename STDATA>
    void saveParameters( STDATA& stData) const
          {
            CHECK_MEMBER_TEMPLATE( svt_check::RequireStData<STDATA>);
            stData.setValue( "kernel_type", name());
          };

    static const char* name()
          {
            return "histintersect";
          }

    static const char* description()
          {
            return "histintersect kernel: min(u(i),v(i))";
          }

    /*======================================================================*/
    /*! 
     *   get information about the parameters, that are used in
     *   loadParameters() and saveParameters(). The Infos are appended
     *   to the passed array
     *
     *   \param p  array of ParamInfos. The Infos will be appended to
     *             this array
     */
    /*======================================================================*/
    static void getParamInfos( std::vector<ParamInfo>& p)
          {}
    
    
    
  };
}

#endif
