/**************************************************************************
**       Title: convert plain svmtl input files to distance matrix file
**    $RCSfile: plain_to_dist_float.cc,v $
**   $Revision: 1.1 $$Name:  $
**       $Date: 2005/07/19 13:05:19 $
**   Copyright: GPL $Author: haasdonk $
** Description: converts a given svmtl plain input file to a kernel matrix
**              binary float representation and a global-id file containing the
**              class labels and a reference into the kernel matrix.
**              the structure of the binary float file is:
**              2 unsigned int (each 4 byte) indicating the number of 
**                    rows and columns of the matrix
**              followed by floats representing row-wise the distance matrix.
**              The structure of the gid-ascii-file is row-wise: one numeric 
**              label followed by an integer global index (gid) into the 
**              kmf-file
**              
** The following commands 1. and 2. should give the same results 
** (except the latter being faster, the difference being even more 
**  obvious for larger datasets):
**
** CROSS-VALIDATION:
**
** 1. time ../svmtl/svmtl crossval -v 270 -kf rbf -g 0.1 
**      ../svmtl/data/heart_scale.vec
** 
**  => 1.377u 0.326s 0:02.04 82.8%     0+0k 0+0io 0pf+0w
**
** compared to 
**
** 2. time ./plain_to_dist_float -kf edsqr 
**      -kmf test-ED-rows-1-270-cols-1-270.kmf -gid test-ED.gid 
**      ../svmtl/data/heart_scale.vec
**
** time ../svmtl/svmtl_dist_float crossval -v 270 -kf DS_rbf_file_binary -g 0.1
**     -kmf  test-ED-rows-1-270-cols-1-270.kmf test-ED.gid
**
** =>       0.089u 0.003s 0:00.12 66.6%     0+0k 0+0io 0pf+0w
**    plus  1.115u 0.318s 0:01.52 93.4%     0+0k 0+0io 0pf+0w
**
** GRIDSEARCH:
**
** 1. time ../svmtl/svmtl gridsearch -v 270 -p1 gamma:0.1,mul10,100 -p2 
**         cost:0.1,mul10,100 -v 270 -kf rbf -g 0.1 
**         ../svmtl/data/heart_scale.vec
**
** => 120.202u 2.295s 2:06.20 97.0%   0+0k 0+0io 0pf+0w 
**
** 2. time ./plain_to_dist_float -kf edsqr 
**             -kmf test-ED-rows-1-270-cols-1-270.kmf 
**             -gid test-ED.gid ../svmtl/data/heart_scale.vec
**
** time ../svmtl/svmtl_dist_float gridsearch -v 270 -p1 gamma:0.1,mul10,100 
**      -p2 cost:0.1,mul10,100 -v 270 -kf DS_rbf_file_binary -g 0.1 
**      -kmf test-ED-rows-1-270-cols-1-270.kmf test-ED.gid 
**
** =>       0.089u 0.003s 0:00.12 66.6%     0+0k 0+0io 0pf+0w
**   plus 108.565u 2.348s 1:55.03 96.4%   0+0k 0+0io 0pf+0w
**
**-------------------------------------------------------------------------
**
**  $Log: plain_to_dist_float.cc,v $
**  Revision 1.1  2005/07/19 13:05:19  haasdonk
**  initial revision
**
**
**************************************************************************/

#include <iostream>
#include <fstream>
#include <cerrno>       // errno
#include <cstring>      // strerror()
#include <string>

#include "KernelApplication.hh"
#include <BasicFV.hh>
//#Include <SVMApplication.hh>
//#include <AlgorithmLists.hh>
//#include <DefaultMultiClassList.hh>
//#include <DefaultTwoClassList.hh>
//#include <DefaultOneClassList.hh>
//#include <DefaultKernelList.hh>
#include <Kernel_RBF.hh>
#include <Kernel_POLY.hh>
#include <Kernel_LINEAR.hh>

#include <Kernel_DS_RBF.hh>
#include <Kernel_DS_TRIA.hh>
#include <Kernel_EDsqr.hh>

#include <LoadSaveASCII.hh>

typedef TLIST_6( 
    svt::Kernel_EDsqr,
    svt::Kernel_RBF,
    svt::Kernel_POLY,
    svt::Kernel_LINEAR,
    svt::Kernel_DS_TRIA< svt::Kernel_EDsqr >, 
    svt::Kernel_DS_RBF< svt::Kernel_EDsqr > 
    )
    MyKernelList;

typedef svt::KernelApplication< svt::BasicFV,
                             MyKernelList,
                             svt::LoadSaveASCII>  MyKernelApplication;

int main( int argc, const char** argv)
{
  MyKernelApplication ka;
  return ka.main( argc, argv, std::cout);
}  
