/**************************************************************************
**       Title: convert SparseFV to BasicFV
**    $RCSfile: sparse_to_plain.cc,v $
**   $Revision: 1.2 $$Name:  $
**       $Date: 2004/09/03 09:13:47 $
**   Copyright: GPL $Author: ronneber $
** Description:
**
**    
**
**-------------------------------------------------------------------------
**
**  $Log: sparse_to_plain.cc,v $
**  Revision 1.2  2004/09/03 09:13:47  ronneber
**  - return 0 if all went OK
**
**  Revision 1.1  2004/08/26 08:36:59  ronneber
**  initital import
**
**
**
**************************************************************************/

#include <iostream>
#include <fstream>
#include <cerrno>       // errno
#include <cstring>      // strerror()
#include <string>

#include <SparseFV.hh>
#include <BasicFV.hh>

int main( int argc, char** argv)
{
  if( argc != 3)
  {
    std::cerr << "usage: sparse_to_plain data.sparse data.plain\n\n";
    return 1;
  }
  std::string inFileName = argv[1];
  std::string outFileName = argv[2];
  

  /*-----------------------------------------------------------------------
   *  open input file
   *-----------------------------------------------------------------------*/
  std::ifstream inFile( inFileName.c_str());
  if( !inFile)
  {
    std::cerr << "error: Can't open '" << inFileName << "': " 
              << strerror(errno);
    exit( 1);
  }
  
  /*-----------------------------------------------------------------------
   *  read feature vectors one per line
   *-----------------------------------------------------------------------*/
  std::cout << "Loading feature vectors from '"<< inFileName << "'\n";
  std::vector<svt::SparseFV> sparseFVs;
  while( inFile.good())
  {
    //skip spaces and find eof
    char c;
    while (inFile.good() && inFile.get(c) && isspace(c)) 
    {}
    
    if (!inFile.good())
    {
      break;
    }
    else if (!isspace(c))
    {
      inFile.putback(c);
    }
    
    svt::SparseFV fv;
    inFile >> fv;
    sparseFVs.push_back(fv);
  }

  std::cout << "Found " << sparseFVs.size() << " feature vectors\n";

  /*-----------------------------------------------------------------------
   *  find out maximum index
   *-----------------------------------------------------------------------*/
  std::cout << "Searching for feature vectors maximum size\n";
  unsigned int maxIndex = 0;
  for( std::vector<svt::SparseFV>::const_iterator p = sparseFVs.begin();
       p != sparseFVs.end(); ++p)
  {
    svt::SparseFV::const_iterator q = p->end();
    --q;
    if( q->index() > maxIndex) maxIndex = q->index();
  }
  
  std::cout << "maximum index is: " << maxIndex << "\n";
  
  /*-----------------------------------------------------------------------
   *  Create output file and write out feature vectors in BasicFV format
   *-----------------------------------------------------------------------*/
  std::ofstream outFile( outFileName.c_str());
  if( !outFile)
  {
    std::cerr << "error: Can't create '" << outFileName << "': " << strerror(errno);
    exit( 1);
  }
  
  std::cout << "saving feature vectors to '" << outFileName << "'\n";
  for( std::vector<svt::SparseFV>::const_iterator p = sparseFVs.begin();
       p != sparseFVs.end(); ++p)
  {
    outFile << p->getLabel();
    for( unsigned int i = 1; i <= maxIndex; ++i)
    {
      outFile << " " << (*p)[i];
    }
    outFile << std::endl;
  }
  
  return 0;
}

  
