/**************************************************************************
**       Title: compare svmtl to original libsvm
**    $RCSfile: compareToOrigLibsvm.cc,v $
**   $Revision: 1.5 $$Name:  $
**       $Date: 2004/09/09 14:43:50 $
**   Copyright: GPL $Author: ronneber $
** Description:
**
**    
**
**-------------------------------------------------------------------------
**
**  $Log: compareToOrigLibsvm.cc,v $
**  Revision 1.5  2004/09/09 14:43:50  ronneber
**  *** empty log message ***
**
**  Revision 1.4  2004/09/08 14:45:01  ronneber
**  *** empty log message ***
**
**  Revision 1.3  2004/09/03 09:14:35  ronneber
**  *** empty log message ***
**
**  Revision 1.2  2004/09/01 14:45:39  ronneber
**  made compilable
**
**  Revision 1.1  2004/09/01 09:11:00  ronneber
**  added compareToOrigLibsvm
**
**
**
**************************************************************************/

#include <iostream>
#include <fstream>
#include "lmbunit.hh"


static const std::string PWD = TOP_SRC_DIR "/src/test/";
static const std::string SVM_TRAIN_PROG = TOP_BUILD_DIR "/src/test/original-libsvm-2.6/svm-train";
static const std::string SVM_PREDICT_PROG = TOP_BUILD_DIR "/src/test//original-libsvm-2.6/svm-predict";
static const std::string SVMTL_PROG = TOP_BUILD_DIR "/src/svmtl/svmtl";
static const std::string SPARSE_TO_PLAIN_PROG = TOP_BUILD_DIR "/src/sparse_to_plain/sparse_to_plain";


static void testHeartScale()
{
  /*-----------------------------------------------------------------------
   *  Linear training with original libsvm
   *-----------------------------------------------------------------------*/
  std::string cmd = SVM_TRAIN_PROG + " -t 0 " 
      + PWD + "original-libsvm-2.6/heart_scale "
      "/tmp/orig_linear.model "
      "> /tmp/output1a.txt";
  LMBUNIT_DEBUG_STREAM << "executing: " << cmd << std::endl;
  int status = system( cmd.c_str());
  LMBUNIT_ASSERT_EQUAL( status, 0);
  
  /*-----------------------------------------------------------------------
   *  classification with original libsvm
   *-----------------------------------------------------------------------*/
  cmd = SVM_PREDICT_PROG + " " 
      + PWD + "original-libsvm-2.6/heart_scale "
      "/tmp/orig_linear.model "
      "/tmp/orig_linear.out "
      "> /tmp/output1b.txt";
  LMBUNIT_DEBUG_STREAM << "executing: " << cmd << std::endl;
  status = system( cmd.c_str());
  LMBUNIT_ASSERT_EQUAL( status, 0);
  
  /*-----------------------------------------------------------------------
   *  convert heart_scale dataset to svmtl format
   *-----------------------------------------------------------------------*/
  cmd = SPARSE_TO_PLAIN_PROG + " " 
      + PWD + "original-libsvm-2.6/heart_scale "
      "/tmp/heart_scale_svmtl "
      "> /tmp/output1c.txt";
  LMBUNIT_DEBUG_STREAM << "executing: " << cmd << std::endl;
  status = system( cmd.c_str());
  LMBUNIT_ASSERT_EQUAL( status , 0);
  
  /*-----------------------------------------------------------------------
   *  linear training with svmtl
   *-----------------------------------------------------------------------*/
  cmd = SVMTL_PROG + " train "
      "--draw_progress_bar 0 "
      "--verbose_level 0 "
      "--kernel_type linear "
      "--multi_class_type one_vs_one "
      "--modelfile "
      "/tmp/svmtl_linear.model "
      "/tmp/heart_scale_svmtl "
      "> /tmp/output1d.txt";
  LMBUNIT_DEBUG_STREAM << "executing: " << cmd << std::endl;
  status = system( cmd.c_str());
  LMBUNIT_ASSERT_EQUAL( status , 0);
  
  /*-----------------------------------------------------------------------
   *  classification with svmtl
   *-----------------------------------------------------------------------*/
  cmd = SVMTL_PROG + " classify "
      "--draw_progress_bar 0 "
      "--verbose_level 0 "
      "--modelfile "
      "/tmp/svmtl_linear.model "
      "--outfile "
      "/tmp/svmtl_linear.out "
      "/tmp/heart_scale_svmtl "
      "> /tmp/output1e.txt";
  LMBUNIT_DEBUG_STREAM << "executing: " << cmd << std::endl;
  status = system( cmd.c_str());
  LMBUNIT_ASSERT_EQUAL( status , 0);
  
  /*-----------------------------------------------------------------------
   *   test if result files were created
   *-----------------------------------------------------------------------*/
  std::ifstream resultFile1( "/tmp/orig_linear.out");
  std::ifstream resultFile2( "/tmp/svmtl_linear.out");
  
  LMBUNIT_ASSERT( resultFile1);
  LMBUNIT_ASSERT( resultFile2);
  
  /*-----------------------------------------------------------------------
   *  check if results are equal
   *-----------------------------------------------------------------------*/
  int label1, label2;
  while( resultFile1 >> label1)
  {
    resultFile2 >> label2;
    LMBUNIT_ASSERT_EQUAL( label1, label2);
  }
  
  
}

static void testDataScalingWithoutAlgorithm()
{
  std::string cmd = SVMTL_PROG + " train "
      "--draw_progress_bar 0 "
      "--verbose_level 0 "
      "--kernel_type kmatrix_scaled_linear " 
      "--modelfile "
      "/tmp/model.svmtl " 
      "/tmp/heart_scale_svmtl "
      "> /tmp/output2a.txt 2>&1";
  
  LMBUNIT_DEBUG_STREAM << "executing: " << cmd << std::endl;
  int status = system( cmd.c_str());
  LMBUNIT_ASSERT( status != 0);
}

static void testDataScaling()
{
  std::string cmd = SVMTL_PROG + " train "
      "--draw_progress_bar 0 "
      "--verbose_level 0 "
      "--kernel_type kmatrix_scaled_linear " 
      "--scale_algorithm minmax " 
      "--modelfile "
      "/tmp/modelscale.model "
      "/tmp/heart_scale_svmtl "
      "> /tmp/output2b.txt";
  
  LMBUNIT_DEBUG_STREAM << "executing: " << cmd << std::endl;
  int status = system( cmd.c_str());
  LMBUNIT_ASSERT( status == 0);

  /*-----------------------------------------------------------------------
   *  check if modelfile was generated
   *-----------------------------------------------------------------------*/
  std::ifstream modelFile( "/tmp/modelscale.model");
  LMBUNIT_ASSERT_EQUAL( !!modelFile, true);
  

}


int main( int argc, char** argv)
{

  LMBUNIT_WRITE_HEADER();
  
  LMBUNIT_RUN_TEST( testHeartScale()); 
  LMBUNIT_RUN_TEST( testDataScalingWithoutAlgorithm()); 
  LMBUNIT_RUN_TEST( testDataScaling() ); 

  LMBUNIT_WRITE_STATISTICS();

  return _nFails;
}
